%diffractor
% GUI to demonstrate the parameters which control the far-field
% (Fraunhofer) diffraction pattern of a grading of N slits or width w, with
% slit separation s.
%
% LAST UPDATED by Andy French January 2016

function varargout = diffractor(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @diffractor_OpeningFcn, ...
    'gui_OutputFcn',  @diffractor_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% Harmonograph opening function. This executes just before diffractor is made visible.
function diffractor_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for diffractor
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;

%Center figure
figcentre( d.h.FIGUREdiffractor );

%Set defaults
diffractor_defaults

%Update diffractor
diffractor_update;

%%

% Default parameters
function diffractor_defaults
global d

%Wavelength /nm

%http://science-edu.larc.nasa.gov/EDDOCS/Wavelengths_for_Colors.html
d.lamda = 400;  %Violet
d.lamda = 445;  %Indigo
d.lamda = 475;  %Blue
d.lamda = 510;  %Green
d.lamda = 570;  %Yellow
d.lamda = 590;  %Orange
d.lamda = 650;  %Red

%Speed of light /ms^-1
d.c = 2.998e8;

%Frequency /Hz
d.f = d.c/(d.lamda*1e-9);

%Max number of slits and default number of slits
d.Nmax = 100;
d.N = 10;

%Slit spacing /wavelength
d.smax = 20;
d.s = 3;

%Slit width /wavelength
d.wmax = 5;
d.w = 1;

%Fontsize for graphs
d.fsize = 18;

%Default is dB plot is off, i.e. plot power instead
d.dB = 0;

%Plot externally and save .PNG
d.plot_and_save_PNG = 0;

%Set GUI parameters
set( d.h.EDITlamda,'string',num2str(d.lamda) );
set( d.h.EDITc,'string',num2str(d.c) );
set( d.h.EDITf,'string',num2str(d.f/1e12) );
set( d.h.EDITN,'string',num2str(d.N) );
set( d.h.EDITs,'string',num2str(d.s) );
set( d.h.EDITw,'string',num2str(d.w) );
set( d.h.SLIDERN,'value',( d.N - 1 )/(d.Nmax-1) );
set( d.h.SLIDERs,'value',d.s/d.smax );
set( d.h.SLIDERw,'value',d.w/d.wmax);

%%

%% Diffractor output function. Outputs from this function are returned to the command line.
function varargout = diffractor_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Toggle dB for plot
function TOGGLEdB_Callback(hObject, eventdata, handles)
global d
if d.dB==1
    d.dB = 0;
else
    d.dB = 1;
end
diffractor_update

%%

%% Save PNG of graph
function PUSHsavepng_Callback(hObject, eventdata, handles)
global d
d.plot_and_save_PNG = 1;
diffractor_update;
d.plot_and_save_PNG = 0;

%%

%% Number of slits slider
function SLIDERN_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERN,'value');

%Determine vale of N and update associated edit box
d.N = 1 + v * ( d.Nmax - 1 );
d.N = round(d.N);
set(d.h.EDITN,'string',num2str(d.N));

%Update GUI
diffractor_update

%%

%% Slit spacing /wavelength slider
function SLIDERs_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERs,'value');

%Determine vale of N and update associated edit box
d.s = v * d.smax;
set(d.h.EDITs,'string',num2str(d.s));

%Check limits and clip if necessary
if d.s < 0.001
    d.s = 0.001;
end

%Update GUI
diffractor_update

%%

%Wavelnegth /nm edit box
function EDITlamda_Callback(hObject, eventdata, handles)
global d
d.lamda = str2num( get( d.h.EDITlamda,'string') );
diffractor_update

%%

%Speed of light editbox
function EDITc_Callback(hObject, eventdata, handles)
global d
d.c = str2num( get( d.h.EDITc,'string') );
diffractor_update

%%

%% Slit width /wavelength slider
function SLIDERw_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERw,'value');

%Determine vale of N and update associated edit box
d.w = v * d.wmax;
set(d.h.EDITw,'string',num2str(d.w));

%Check limits and clip if necessary
if d.w < 0.001
    d.w = 0.001;
end

%Update GUI
diffractor_update

%%

%% Number of slits edit box
function EDITN_Callback(hObject, eventdata, handles)
global d

%Get numerical value of N
d.N = str2num( get(handles.EDITN,'string') );

%Check limits and clip if necessary
if d.N > d.Nmax
    d.N = d.Nmax;
elseif d.N < 1
    d.N = 1;
end
d.N = round(d.N);

%Update EDIT box
set(handles.EDITN,'string',num2str( d.N ) );

%Update slider
v = ( d.N - 1 ) ...
    / ( d.Nmax - 1);
set(handles.SLIDERN,'value',v);

%Update Harmonograph
diffractor_update

%%

%% Slit spacing edit box
function EDITs_Callback(hObject, eventdata, handles)
global d

%Get numerical value of s
d.s = str2num( get(handles.EDITs,'string') );

%Check limits and clip if necessary
if d.s > d.smax
    d.s = d.smax;
elseif d.s < 0.001
    d.s = 0.001;
end

%Update EDIT box
set(handles.EDITs,'string',num2str( d.s ) );

%Update slider
v = d.s/d.smax;
set(handles.SLIDERs,'value',v);

%Update Harmonograph
diffractor_update

%%

%% Slit width edit box
function EDITw_Callback(hObject, eventdata, handles)
global d

%Get numerical value of s
d.w = str2num( get(handles.EDITw,'string') );

%Check limits and clip if necessary
if d.w > d.wmax
    d.w = d.wmax;
elseif d.w < 0.001
    d.w = 0.001;
end

%Update EDIT box
set(handles.EDITw,'string',num2str( d.w ) );

%Update slider
v = d.w/d.wmax;
set(handles.SLIDERw,'value',v);

%Update Harmonograph
diffractor_update

%%

%Update GUI
function diffractor_update
global d

%Wavelength /m
lamda = d.lamda*1e-9;

%Wavenumber /m^-1
k = 2*pi/lamda;

%Slit width /m
w = d.w*lamda;

%Slit spacing /m
s = d.s*lamda;

%Calculate frequency /THz and update GUI
f = (d.c/lamda)/1e12;
set( d.h.EDITf,'string',num2str(f) );

%Calculate far-field range limit /m
r_far_field = ( (d.N*s)^2 )/lamda;

%Set angles
theta = linspace( -pi/2,pi/2,1000);

%Determine intensity pattern amplitude variation with angle (i.e. range is
%the same)
I_s = ( sinN( 0.5*k*s*sin(theta),d.N) ).^2;
I_w = ( sinc( 0.5*k*w*sin(theta))  ).^2;
I = I_s.*I_w;

if d.plot_and_save_PNG==1
    %Create new figure
    fig = figure;
    figure(fig);
else
    %Point to axes
    figure( d.h.FIGUREdiffractor );
    axes( d.h.AXESdiffractor );
    cla;
end

if d.dB==1
    %Plot intensity pattern (i.e. wave power)in dBm
    plot( theta*180/pi, dBm(I) );
    xlabel( '\theta^o', 'fontsize',d.fsize );
    ylabel('Normalized wave power /dBm','fontsize',d.fsize )
    title({'Grating Fraunhofer far field diffraction',['\lambda = ',...
        num2str(lamda/1e-9),'nm,  s = ',num2str(s/lamda),...
        '\lambda, w = ',num2str(w/lamda),'\lambda, N = ',num2str(d.N)]},'fontsize',d.fsize);
    xlim([-90,90])
    ylim([-60,0])
    grid on;
    set(gca,'fontsize',d.fsize);
    
else
    %Plot intensity pattern (i.e. wave power)
    plot( theta*180/pi, I_w,'r--' );
    hold on;
    plot( theta*180/pi, I,'b-' );
    xlabel( '\theta^o', 'fontsize',d.fsize );
    ylabel('Normalized wave power','fontsize',d.fsize )
    title({'Grating Fraunhofer far field diffraction',['\lambda = ',...
        num2str(lamda/1e-9),'nm,  s = ',num2str(s/lamda),...
        '\lambda, w = ',num2str(w/lamda),'\lambda, N = ',num2str(d.N)]},'fontsize',d.fsize);
    xlim([-90,90])
    ylim([0,1])
    grid on;
    set(gca,'fontsize',d.fsize);
end

%Save externally plotted graph, if option set
if d.plot_and_save_PNG==1
    png_filename = [ 'Fraunhofer dB = ',num2str(d.dB),' lamda = ',...
        num2str(d.lamda),'  s = ',num2str(s/lamda),' w = ',num2str(w/lamda),' N = ',num2str(d.N),'.png'];
    print( fig,png_filename,'-dpng','-r300');
    close(fig);
    figure(d.h.FIGUREdiffractor); axes(d.h.AXESdiffractor);
end


%%

function y = sinc(x)
y = ones(size(x));
k = x~=0;
y(k) = sin(x(k))./x((k));

%%

function y = sinN(x,N)
y = N*ones(size(x));
k = x~=0;
y(k) = sin(N*x(k))./(N*sin(x(k)));

%%

function y = dBm(x)
y = NaN(size(x));
k = x>0;
y(k) = 10*log10(x);
y = y - max(y);

%%

%figcenter
% Centres a figure with handle fig in the current display.

function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');
display_size=get(0,'MonitorPositions');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*( display_size(3)-fig_position(3) ) - screen_size(1);
centre_y = 0.5*( display_size(4)-fig_position(4) ) - screen_size(2);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code






